import { reactive, computed, nextTick } from 'vue'
import { ServeTalkRecords } from '@/api/chat'
import { useDialogueStore, useDialogueListStore } from '@/store'
import { formatTalkRecord } from '@/utils/talk'
import { addClass, removeClass } from '@/utils/dom'
import lodash from 'lodash'

export const useTalkRecord = (uid) => {
  const dialogueStore = useDialogueStore()
  const { getDialogueList,addDialogueRecord,zpagingComplete } = useDialogueListStore()
  const records = computed(() => {
    const dialogueList = getDialogueList(dialogueStore.index_name)
    if(dialogueList){
      return lodash.cloneDeep(dialogueList.records).reverse()
    }
    return lodash.cloneDeep(dialogueStore.records)?.reverse()
  } )

  const location = reactive({
    msgid: '',
    num: 0
  })

  const loadConfig = reactive({
    receiver_id: 0,
    talk_type: 0,
    status: 0,
    cursor: 0
  })

  const onJumpMessage = (msgid) => {
    const element = document.getElementById(msgid)
    if (!element) {
      if (location.msgid == '') {
        location.msgid = msgid
        location.num = 3
      } else {
        location.num--

        if (location.num === 0) {
          location.msgid = ''
          location.num = 0
          message.warning('仅支持查看最近300条的记录')
          return
        }
      }

      const el = document.getElementById('imChatPanel')

      return el?.scrollTo({
        top: 0,
        behavior: 'smooth'
      })
    }

    location.msgid = ''
    location.num = 0

    element?.scrollIntoView({
      behavior: 'smooth'
    })

    addClass(element, 'border')

    setTimeout(() => {
      element && removeClass(element, 'border')
    }, 3000)
  }

  // 加载数据列表
  const load = async (params) => {
    const request = {
      limit:30,
      ...params,
      talk_type: params.talk_type,
      receiver_id: params.receiver_id,
      cursor: loadConfig.cursor,
    }

    loadConfig.status = 0

    let scrollHeight = 0
    const el = document.getElementById('imChatPanel')
    // if (el) {
    //   scrollHeight = el.scrollHeight
    // }
    const { data, code } = await ServeTalkRecords(request)
    if (code != 200) {
      return (loadConfig.status = 1)
    }

    // 防止对话切换过快，数据渲染错误
    if (
      request.talk_type != loadConfig.talk_type ||
      request.receiver_id != loadConfig.receiver_id
    ) {
      return (location.msgid = '')
    }
    const items = (data.items || []).map((item) => formatTalkRecord(uid, item))
    if (request.cursor == 0) {
      // 判断是否是初次加载
      dialogueStore.clearDialogueRecord()
    }

    const reverseItems = lodash.cloneDeep(items).reverse()
    dialogueStore.unshiftDialogueRecord(reverseItems)
    addDialogueRecord(params.direction=='down'?reverseItems:items,params.direction=='down'?'add':'unshift')
    zpagingComplete(dialogueStore.index_name)
    const dialogueList = getDialogueList(dialogueStore.index_name)
    loadConfig.status = dialogueList.records?.[0]?.sequence > 1 ? 1 : 2
    // loadConfig.cursor = data.cursor

    nextTick(() => {
      const el = document.getElementById('imChatPanel')

      if (el) {
        if (request.cursor == 0) {
          el.scrollTop = el.scrollHeight

          setTimeout(() => {
            el.scrollTop = el.scrollHeight + 1000
          }, 50)
        } else {
          el.scrollTop = el.scrollHeight - scrollHeight
        }
      }

      if (location.msgid) {
        onJumpMessage(location.msgid)
      }
    })
  }

  const onRefreshLoad = () => {
    let dialogueList = getDialogueList(dialogueStore.index_name)
    if (dialogueList.records[0].sequence === 1 ) {
      return false
    }
    if (loadConfig.status == 1) {
      let filterList = dialogueList.records.filter(item=>item.sequence !== -1)
      loadConfig.cursor = filterList[0].sequence
      load({
        receiver_id: loadConfig.receiver_id,
        talk_type: loadConfig.talk_type,
        direction:'up',
        no_limit:0,
      })
    }
  }

  const onLoad = async (params) => {
    let dialogueList = getDialogueList(params.index_name)
    if (!dialogueList) {
      loadConfig.cursor = 0
    }else{
      let filterList = dialogueList.records.filter(item=>item.sequence !== -1)
      if(params.direction=='up'){
        loadConfig.cursor = filterList?.[0]?.sequence
      }else{
        loadConfig.cursor = filterList?.[filterList.length-1]?.sequence
      }
    }
    loadConfig.receiver_id = params.receiver_id
    loadConfig.talk_type = params.talk_type

    await load(params)
  }

  return { loadConfig, records, onLoad, onRefreshLoad, onJumpMessage }
}
